<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'CrBcHelpers.php');
require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'CrBcCart.php');

class CrBcProperties
{
    static $BACKORDERS_DISALLOWED_VISIBLE = 'DISALLOWED_VISIBLE';
    static $BACKORDERS_DISALLOWED_HIDDEN = 'DISALLOWED_HIDDEN';
    
    public static function getCombinationSelectors($product_id, $amount = 1, $selected = array() ){

        if(!$amount){
            $amount = 1;
        }

        $out = '<div id="crbc-properties-'.$product_id.'" class="crbc-properties crbc-combination">';

        $rows = self::getCombinationRows($product_id);
        
        $pform = new JRegistry('', JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'models'.DS.'product.xml');

        // used for global override
        $form = CrBcHelpers::getBcConfig();

        $options = array();
        
        $init_selected = array();
        
        $cart = new CrBcCart( JFactory::getSession()->get('crbc_cart', array()) );
        foreach($rows As $row){
            
            if (!isset($options[$row->id])) {
                $options[$row->id] = array();
                $options[$row->id]['ppc_id'] = $row->id;
                $options[$row->id]['ppi_id'] = $row->ppi_id;
                $options[$row->id]['name'] = $row->name;
                $options[$row->id]['price'] = $row->price;
                $options[$row->id]['stock'] = $row->stock;
                $options[$row->id]['producttaxclass_id'] = $row->producttaxclass_id;
                $options[$row->id]['virtual_product'] = $row->virtual_product;
                $options[$row->id]['product_id'] = $row->pid;
                $options[$row->id]['attribs'] = $row->attribs;
                $options[$row->id]['selected'] = $row->selected;
            }

            $pform->loadString($options[$row->id]['attribs']);

            $disallowed = false;
                    
            if( !$options[$row->id]['virtual_product']
                    &&
                    $options[$row->id]['stock'] <= 0
                    &&
                    $pform->get('backorders','') != ''
                 ){
                if( $pform->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN ){
                      $disallowed = true;
                }

            } else if( !$options[$row->id]['virtual_product']
                    &&
                    $options[$row->id]['stock'] <= 0
                    &&
                    $form->get('backorders','') != ''
                 ){
                if( $form->get('backorders','') == self::$BACKORDERS_DISALLOWED_HIDDEN ){
                      $disallowed = true;
                }
            }

            if(!$disallowed){
                
                if(count($selected) == 0){
                    
                    if( $row->selected ){
                        $init_selected[] = $row->id;
                    }
                }
                
                $options[$row->id]['options'][] = $row->id;
                
                if($row->custom_value == ''){
                    
                    // values translations
                    $translation = CrBcHelpers::loadTranslation($row->property_values_id, 'property_value_'.$row->property_values_key_id);

                    if(is_object($translation)){

                        $options[$row->id]['values'][] = $translation->title;
                    }
                    else
                    {
                        $options[$row->id]['values'][] = $row->value;
                    }
                }
                else
                {
                    
                    // values translations
                    $translation = CrBcHelpers::loadTranslation($row->productpropertyvalue_id, 'productpropertyvalue');

                    if(is_object($translation)){

                        $options[$row->id]['values'][] = $translation->body;
                    }
                    else
                    {
                        $options[$row->id]['values'][] = $row->custom_value;
                    }
                }
                
                // keys translations
                $translation = CrBcHelpers::loadTranslation($row->property_keys_id, 'property_key_'.$row->property_id);
                
                if(is_object($translation)){
                    
                    $options[$row->id]['keys'][] = $translation->title;
                }
                else
                {
                    $options[$row->id]['keys'][] = $row->property_key;
                }
                
            }else{
                unset($options[$row->id]);
            }
        }
        
        if(count($selected) == 0){
            $selected = $init_selected;
        }

        $select_open = '';
        $select_close = '';
        $option_str = '';

        if(count($options)){
            $out .= '<h3 id="crbcPropertiesTitle">'.JText::_('COM_BREEZINGCOMMERCE_COMBINATIONS').'</h3>'."\n";
            $out .= '<div class="crbc-well well">';
        }

        JFactory::getDBO()->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'productprice' Order By `ordering`");
        $plugins = JFactory::getDBO()->loadObjectList();

        // just in case we need the customer id for anything regarding prices
        
        $customer_id = 0;

        if($cart instanceof CrBcCart && JFactory::getUser()->get('id', 0) != 0){
            
            JFactory::getDbo()->setQuery("Select id From #__breezingcommerce_customers Where userid = " . intval(JFactory::getUser()->get('id', 0)) . " Limit 1");
            $customer_id = JFactory::getDbo()->loadResult();

        } else {
            
            $customer_id = isset($cart->cart['customer_id']) ? $cart->cart['customer_id'] : 0;
        }
        
        $i = 0;
        
        // determine the id for the element, consisting of all ppc ids within a combination
        $ppc_id_str = '';
        foreach($options As $key => $opts){
            $ppc_id_str .= $opts['ppc_id'];
        }
        
        foreach($options As $key => $opts){
            
            if($i == 0){
                $select_open .= '<div id="crbcPropertiesWrapper'.$ppc_id_str.'" class="crbcPropertiesWrapper">'."\n";
                $select_open .= '<select onchange="crbcUpdatePriceByProperty(); crbcApplyCombinationDisplayPlugins(this);" name="crbcProperties[]" id="crbcProperties'.$ppc_id_str.'" class="crbcProperties crbcCombinations">'."\n";
            }

            $option_str .= '<option value="' . $opts['ppc_id'] . '"' . (in_array($opts['ppc_id'], $selected) ? ' selected="selected"' : '') . '>';
            $j = 0;
            $size = count($opts['options']);
            foreach ($opts['options'] As $opt) {
                //if($j == 0 && $opts['name'] != ''){
                //    $option_str .= $opts['name'] . ': ';
                //}
                if ($opts['keys'][$j] != '') {
                    $option_str .= htmlentities($opts['keys'][$j], ENT_QUOTES, 'UTF-8') . ': ';
                }
                $option_str .= htmlentities($opts['values'][$j], ENT_QUOTES, 'UTF-8');
                if ($j + 1 < $size) {
                    $option_str .= ', ';
                }
                $j++;
            }
            $price = '';
            $new_price = $opts['price'];
            foreach ($plugins As $plugin) {
                $class = CrBcPlugin::getPluginClass($plugin->name, $plugin->type);
                $class_instance = new $class();
                $new_price = $class::getPrice($opts['product_id'], $opts['ppc_id'], $amount, $new_price < 0 ? 0 : $new_price);
                if ($new_price !== null) {
                    $opts['price'] = $new_price;
                }
            }
            if ($opts['price'] > 0) {
                
                $price = CrBcCart::getItemPrice($opts['price'], $cart->currency_conversion_rate, $opts['producttaxclass_id'], array(), array(), $amount, $customer_id);
                $price = ' (' . $cart->formatPrice($price['gross']) . ')';
            }
            $option_str .= $price . '</option>' . "\n";
            
            if($i == 0){
                $select_close .= '</select>'."\n";
                $select_close .= '</div>'."\n";
            }

            $i++;
        }

        $out .= $select_open . $option_str . $select_close;

        if(count($options)){
            $out .= '</div>'; // well end
        }
        
        $out .= '</div>';
        
        return $out;
    }

    public static function getCombinationRows($product_id){
        $db = JFactory::getDBO();
        $db->setQuery("
            Select Distinct
                ppc.*,
                ppi.id As ppi_id,
                pv.`value`,
                ppv.id As productpropertyvalue_id,
                ppv.custom_value,
                pv.id As property_values_id,
                pv.key_id As property_values_key_id,
                k.key As property_key,
                k.property_id As property_keys_property_id,
                k.id As property_keys_id,
                p.producttaxclass_id,
                p.virtual_product,
                p.id As pid,
                p.attribs
            From
                #__breezingcommerce_product_property_combinations As ppc,
                #__breezingcommerce_product_property_combination_items As ppi,
                #__breezingcommerce_product_property_values As ppv,
                #__breezingcommerce_product_property_keys As ppk,
                #__breezingcommerce_property_values As pv,
                #__breezingcommerce_property_keys As k,
                #__breezingcommerce_properties As pr,
                #__breezingcommerce_products As p
            Where
                    k.property_id = pr.id
                And
                    pv.property_id = pr.id
                And
                    pr.readonly = 0
                And
                    ppi.product_id = " . intval($product_id) . "
                And
                    ppc.id = ppi.combination_id
                And
                    ppi.property_key_id = k.id
                And
                    ppv.property_value_id = ppi.property_value_id
                And
                    ppk.property_key_id = ppi.property_key_id
                And
                    ppv.product_id = " . intval($product_id) . "
                And
                    ppk.product_id = " . intval($product_id) . "
                And
                    pv.id = ppi.property_value_id
                And
                    p.id = " . intval($product_id) . "
                And
                    ppc.published = 1
                And
                    ppv.published = 1
                And
                    ppk.published = 1
                Order By
                    ppc.`ordering`,
                    ppk.`ordering`,
                    ppv.`ordering`

        ");

        return $db->loadObjectList();
    }

    public static function getPropertyRows($product_id, $readonly = false){
        $db = JFactory::getDBO();
        $db->setQuery("
                Select Distinct
                    ppv.*,
                    ppk.id As ppk_id,
                    ppk.required As required,
                    pv.`value`,
                    pv.id As property_values_id,
                    pv.key_id As property_values_key_id,
                    pv.property_id As property_values_property_id,
                    k.id As property_keys_id,
                    k.property_id As property_keys_property_id,
                    k.key As property_key,
                    p.producttaxclass_id
                From
                    #__breezingcommerce_product_property_values As ppv,
                    #__breezingcommerce_product_property_keys As ppk,
                    #__breezingcommerce_property_values As pv,
                    #__breezingcommerce_property_keys As k,
                    #__breezingcommerce_properties As pr,
                    #__breezingcommerce_products As p
                Where
                     k.property_id = pr.id
                And
                    pv.property_id = pr.id
                And
                    pr.readonly = " . ( $readonly ? 1 : 0 ) . "
                And
                    ppv.property_key_id = k.id
                And
                    ppk.property_key_id = k.id
                And
                    ppv.published = 1
                And
                    ppk.published = 1
                And
                    ppv.property_key_id = k.id
                And
                    ppk.property_key_id = k.id
                And
                    ppv.product_id = " . intval($product_id) . "
                And
                    ppk.product_id = " . intval($product_id) . "
                And
                    p.id = " . intval($product_id) . "
                And
                    pv.id = ppv.property_value_id
                And
                    k.id = ppk.property_key_id
                Order By
                    ppk.`ordering`,
                    ppv.`ordering`");

        return $db->loadObjectList();
    }
    
    public static function getPropertyJs(){
        return '
        
        function crbcUpdatePriceByProperty(){
            if( typeof crbc_products != "undefined" ){
                for(var i = 0; i < crbc_products.length; i++){
                    crbc_products[i].update_price();
                }
            }
            else if( typeof crbc_product != "undefined" ){
                crbc_product.update_price();
            }
        }
        
        function crbcApplyCombinationDisplayPlugins(the_select){
            if( typeof crbc_products != "undefined" ){
                for(var i = 0; i < crbc_products.length; i++){
                    if(jQuery(the_select).closest("form").attr("id")){
                        var splitted = jQuery(the_select).closest("form").attr("id").split("crbc-order-form-");
                        if(splitted.length == 2 && splitted[1] == crbc_products[i].product_id){
                            crbc_products[i].apply_display_plugins(the_select);
                            break;
                        }
                    }
                }
            }
            else if( typeof crbc_product != "undefined" ){
                crbc_product.apply_display_plugins(the_select);
            }
        }
        
        jQuery(document).ready(function(){
        
            jQuery(".crbc-category-product-entry").each(function(){
            
                var has_combination = jQuery(this).find(".crbcCombinations").size();
                var has_properties = jQuery(this).find(".crbcProperties").size();
                
                if(has_combination > 0){
                    var the_select = jQuery(this).find(".crbcCombinations").get(0);
                    if( typeof crbc_products != "undefined" ){
                        for(var i = 0; i < crbc_products.length; i++){
                            if(jQuery(the_select).closest("form").attr("id")){
                                var splitted = jQuery(the_select).closest("form").attr("id").split("crbc-order-form-");
                                if(splitted.length == 2 && splitted[1] == crbc_products[i].product_id){
                                    crbc_products[i].apply_display_plugins(the_select);
                                    break;
                                }
                            }
                        }
                    }
                    else if( typeof crbc_product != "undefined" ){
                    
                        crbc_product.apply_display_plugins(jQuery(this).find(".crbcCombinations").get(0));
                    }
                }
                else if( has_properties > 0 )
                {
                    var the_select = jQuery(this).find(".crbcProperties").get(0);
                    if( typeof crbc_products != "undefined" ){
                        for(var i = 0; i < crbc_products.length; i++){
                            if(jQuery(the_select).closest("form").attr("id")){
                                var splitted = jQuery(the_select).closest("form").attr("id").split("crbc-order-form-");
                                if(splitted.length == 2 && splitted[1] == crbc_products[i].product_id){
                                    crbc_products[i].apply_display_plugins();
                                    break;
                                }
                            }
                        }
                    }
                    else if( typeof crbc_product != "undefined" ){
                        crbc_product.apply_display_plugins();
                    }
                }
            });
        });
        ';
    }

    public static function getPropertySelectors($product_id, $amount = 1, $selected = array(), $use_combinations = false){
        
        if(!$amount){
            $amount = 1;
        }

        if($use_combinations){
            return self::getCombinationSelectors($product_id, $amount, $selected);
        }
        
        $out = '<div id="crbc-properties-'.$product_id.'" class="crbc-properties">';

        $rows = self::getPropertyRows($product_id);
        
        $options = array();

        require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'classes'.DS.'CrBcCart.php');
        foreach($rows As $row){
            if(!isset($options[$row->property_key])){
                $options[$row->property_key] = array();
                $options[$row->property_key]['ppk_id'] = $row->ppk_id;
                $options[$row->property_key]['required'] = $row->required;
                $options[$row->property_key]['property_keys_id'] = $row->property_keys_id;
                $options[$row->property_key]['property_key'] = 'property_key_'.$row->property_id;
            }
            
            $price = '';
            if($row->price > 0){
                
                $cart = new CrBcCart( JFactory::getSession()->get('crbc_cart', array()) );
                
                // just in case we need the customer id for anything regarding prices
        
                $customer_id = 0;

                if($cart instanceof CrBcCart && JFactory::getUser()->get('id', 0) != 0){

                    JFactory::getDbo()->setQuery("Select id From #__breezingcommerce_customers Where userid = " . intval(JFactory::getUser()->get('id', 0)) . " Limit 1");
                    $customer_id = JFactory::getDbo()->loadResult();

                } else {

                    $customer_id = isset($cart->cart['customer_id']) ? $cart->cart['customer_id'] : 0;
                }
                
                $plus = '';
                if($row->is_plus_price){
                    $plus = '+';
                }
                $price = CrBcCart::getItemPrice( $row->price, $cart->currency_conversion_rate, $row->producttaxclass_id, array(), array(), $amount, $customer_id);
                $price = ' ('.$plus.$cart->formatPrice($price['gross']).')';
            }
            
            if($row->custom_value == ''){
            
                $translation = CrBcHelpers::loadTranslation($row->property_values_id, 'property_value_'.$row->property_values_key_id);
                if(is_object($translation)){
                    $row->value = $translation->title;
                }
            
            }
            else
            {
                $row->value = $row->custom_value;
                        
                $translation = CrBcHelpers::loadTranslation($row->id, 'productpropertyvalue');
                if(is_object($translation)){
                    $row->value = $translation->body;
                }
            }
            
            $options[$row->property_key]['options'][] = '<option value="'.$row->id.'"'.(in_array($row->id, $selected) || (!count($selected) && $row->selected) ? ' selected="selected"' : '').'>'.htmlentities($row->value, ENT_QUOTES, 'UTF-8').$price.'</option>'."\n";
        }

        if(count($options)){
            $out .= '<h3 id="crbcPropertiesTitle">'.JText::_('COM_BREEZINGCOMMERCE_PRODUCT_OPTIONS').'</h3>'."\n";
            $out .= '<div class="crbc-well well">';
        }

        foreach($options As $key => $opts){
            
            $translation = CrBcHelpers::loadTranslation($opts['property_keys_id'], $opts['property_key']);
            if(is_object($translation)){
                $key = $translation->title;
            }
            
            $out .= '<div id="crbcPropertiesWrapper'.$opts['ppk_id'].'" class="crbcPropertiesWrapper">'."\n";
            $out .= '<label for="crbcProperties'.$opts['ppk_id'].'" id="crbcPropertiesLabel'.$opts['ppk_id'].'" class="crbcPropertiesLabel">'.$key.'</label>'."\n";
            $out .= '<select onchange="crbcUpdatePriceByProperty();" name="crbcProperties[]" id="crbcProperties'.$opts['ppk_id'].'" class="crbcProperties">'."\n";
            if(!$opts['required']){
                $out .= '<option value="0">'.JText::_('COM_BREEZINGCOMMERCE_PLEASE_CHOOSE').'</option>'."\n";
            }
            foreach($opts['options'] As $opt){
                $out .= $opt;
            }
            $out .= '</select>'."\n";
            $out .= '</div>'."\n";
        }
        
        if(count($options)){
            $out .= '</div>'; // well end
        }

        $out .= '</div>';
        
        return $out;
    }

    public static function getPropertySets($readonly = false){
        $db = JFactory::getDBO();
        $db->setQuery("Select * From #__breezingcommerce_properties Where readonly = ".($readonly ? 1 : 0)." Order By `ordering`");
        return $db->loadObjectList();
    }

    public static function getPropertyOptions($product_id, $property_id, $use_combinations = 0, $readonly = false)
    {
        $id_infix = $readonly ? '_readonly' : '';
        
        $options = '';
        $property_sets = '';
        $sets = self::getPropertySets($readonly);
        if(count($sets))
        {
            $property_sets = '<select name="property'.$id_infix.'_id" id="property'.$id_infix.'_sets" class="chzn-done">'."\n";
            $property_sets .= '<option value="0">'.JText::_('COM_BREEZINGCOMMERCE_SELECT_PROPERTY_SET').'</option>'."\n";
            foreach($sets As $set){
                $property_sets .= '<option value="'.$set->id.'"'.($set->id == $property_id ? ' selected="selected"' : '').'>'.htmlentities($set->name, ENT_QUOTES, 'UTF-8').'</option>'."\n";
            }
            $property_sets .= '</select>'."\n";
            $options .= $property_sets;
        }

        if($property_id > 0)
        {
            $options .= '<br/><br/><a href="index.php?option=com_breezingcommerce&amp;controller=productproperties&amp;product_id='.$product_id.'&readonly='.($readonly ? 1 : 0).'" title="" class="modal" rel="{handler: \'iframe\', size: {x: 1024, y: 500}}">'.JText::_('COM_BREEZINGCOMMERCE_EDIT_PROPERTIES').'</a>'."\n";
            
            if(!$readonly){
                
                $options .= '<br/><br/><a href="index.php?option=com_breezingcommerce&amp;controller=productpropertycombinations&amp;property_id='.$property_id.'&amp;product_id='.$product_id.'" title="" class="modal" rel="{handler: \'iframe\', size: {x: 1024, y: 500}}">'.JText::_('COM_BREEZINGCOMMERCE_EDIT_PROPERTY_VARIATIONS').'</a>'."\n";
                $options .= '<br/><br/>';

                $options .= '<label>'.JText::_('COM_BREEZINGCOMMERCE_USE_COMBINATIONS').'</label>
                    <fieldset id="use_combinations_fieldset" class="radio btn-group btn-group-yesno">
                        <input type="radio" name="use_combinations" id="use_combinations_1" value="0" '.(!$use_combinations ? ' checked="checked"' : '').'/> <label for="use_combinations_1">'.JText::_('COM_BREEZINGCOMMERCE_NO').'</label>
                        <input type="radio" name="use_combinations" id="use_combinations_2" value="1" '.($use_combinations ? ' checked="checked"' : '').'/> <label for="use_combinations_2">'.JText::_('COM_BREEZINGCOMMERCE_YES').'</label>
                    </fieldset>    
                    ';
            }
        }

        return $options;
    }

    public static function getPropertyCombinationOptions($product_id, $property_id, $combination_id = 0)
    {
        $db = JFactory::getDBO();

        $db->setQuery("Select property_value_id From #__breezingcommerce_product_property_combination_items Where combination_id = ".intval($combination_id));
        $items = $db->loadObjectList();
        $selected_combinations = array();
        foreach($items As $item){
            $selected_combinations[$item->property_value_id] = true;
        }

        $db->setQuery("
            Select
                k.key,
                v.value,
                v.id As value_id,
                k.id As key_id
            From
                #__breezingcommerce_product_property_keys As ppk,
                #__breezingcommerce_product_property_values As ppv,
                #__breezingcommerce_property_keys As k,
                #__breezingcommerce_property_values As v,
                #__breezingcommerce_properties As pr
            Where
                k.property_id = pr.id
            And
                v.property_id = pr.id
            And
                pr.readonly = 0
            And
                ppk.product_id = ".intval($product_id)."
            And
                ppk.property_id = ".intval($property_id)."
            And
                ppv.product_id = ".intval($product_id)."
            And
                ppv.property_id = ".intval($property_id)."
            And
                v.key_id = k.id 
            And
                ppk.property_key_id = k.id
            And
                ppv.property_value_id = v.id
            And
                ppk.published = 1
            And
                ppv.published = 1
            Order By
                ppk.`ordering`,
                ppv.`ordering`,
                v.`ordering`
          ");
        
        $list = $db->loadObjectList();
        
        $out = '';
        $props = array();
        foreach($list As $entry){
            if(!isset($props[$entry->key.':'.$entry->key_id])){
                $props[$entry->key.':'.$entry->key_id] = array();
            }
            $props[$entry->key.':'.$entry->key_id][] = array($entry->value, $entry->value_id);
        }
        $out .= '<table class="table table-striped">';
        foreach($props As $key => $properties){
            // fix the key-name if ":" is used in the key
            $key_id = explode(':', $key);
            if(count($key_id) > 2){
                $fixed_key = '';
                $size = count($key_id);
                for($i = 0; $i < $size-1; $i++){
                    if($i+1 < $size-1){
                        $fixed_key .= $key_id[$i].':';
                    }
                    else
                    {
                        $fixed_key .= $key_id[$i];
                    }
                }
                $id = $key_id[$size-1];
                $key_id = array();
                $key_id[0] = $fixed_key;
                $key_id[1] = $id;
            }
            // finally build options
            $out .= '<tr><td align="left" valign="top" class="key"><label for="crbc-property-combination'.$key_id[1].'">'.$key_id[0].'</label></td>
            <td valign="top">';
            $out .= '<select class="chzn-done" id="crbc-property-combination'.$key_id[1].'" name="property_combination['.$key_id[1].']">'."\n";
            $out .= '<option value="0">'.htmlentities(JText::_('COM_BREEZINGCOMMERCE_SELECT_ONE'), ENT_QUOTES, 'UTF-8').'</option>'."\n";
            foreach($properties As $property){
                $out .= '<option value="'.$property[1].'"'.(isset($selected_combinations[$property[1]]) ? ' selected="selected"' : '').'>'.htmlentities($property[0], ENT_QUOTES, 'UTF-8').'</option>'."\n";
            }
            $out .= '</select>'."\n";
            $out .= '</td></tr>';
        }
        $out .= '</table>';
        
        return $out;
    }

    public static function copyProductProperties($src_id, $dest_id){
        $config['key_id'] = 'id';

        $config['compare_key'] = 'product_id';
        $config['table'] = '#__breezingcommerce_product_property_combinations';
        $combinations = self::copyTableRows($src_id, $dest_id, $config);
        
        $config['compare_key'] = 'combination_id';
        $config['table'] = '#__breezingcommerce_product_property_combination_items';
        
        $size = count($combinations);
        for($i = 0; $i < $size; $i++){
            self::copyTableRows($combinations[$i]['original_id'], $combinations[$i]['insert_id'], $config, array('product_id' => $dest_id));
        }
        
        $config['compare_key'] = 'product_id';
        $config['table'] = '#__breezingcommerce_product_property_keys';
        self::copyTableRows($src_id, $dest_id, $config);

        $config['compare_key'] = 'product_id';
        $config['table'] = '#__breezingcommerce_product_property_values';
        self::copyTableRows($src_id, $dest_id, $config);
    }

    public static function copyTableRows($src_id, $dest_id, $config, $property_overrides = array()){
        $ret = array();
        
        $original_id = 0;
        $insert_ids = array();
        $db = JFactory::getDBO();

        $db->setQuery( ' Select * From '. $config['table'] .
                '  Where '.$config['compare_key'].' = ' . intval($src_id));
        $result = $db->loadObjectList();

        $i = 0;
        foreach($result As $obj)
        {
            foreach($property_overrides As $key => $property_override){
                $obj->{$key} = $property_override;
            }
            
            $ret[$i]['original_id'] = $obj->{$config['key_id']};
            unset($obj->{$config['key_id']});
            $obj->{$config['compare_key']} = intval($dest_id);

            $db->insertObject($config['table'], $obj);
            $ret[$i]['insert_id'] = $db->insertid();
            
            $i++;
        }
        return $ret;
    }

    public static function reorderProperties($product_ids, JModelLegacy $model){

        $keys_updated = 0;
        $db = JFactory::getDBO();

        foreach($product_ids As $product_id){

            $db->setQuery("Select property_id From #__breezingcommerce_products Where id = " . $product_id);
            $property_id = $db->loadResult();

            $db->setQuery("Select id From #__breezingcommerce_property_keys Where property_id = " . $property_id);
            $property_key_ids = $db->loadColumn();

            $model->getTable('productpropertykeys')->reorder('property_id = '.$property_id.' And product_id = ' . $product_id);

            foreach($property_key_ids As $property_key_id){
                $model->getTable('productpropertyvalues')->reorder('property_id = '.$property_id.' And property_key_id = '.$property_key_id.' And product_id = ' . $product_id);
                $keys_updated++;
            }
        }

        return JText::_('COM_BREEZINGCOMMERCE_KEYS_REORDERED') . ': ' . $keys_updated;
    }

    public static function importProperties($product_ids, JModelLegacy $model, $reorder = true, $publish = false, $readonly = false){

        $db = JFactory::getDBO();
        
        $keys_updated = 0;
        $values_updated = 0;
        $published = 0;
        if($publish){
            $published = 1;
        }

        $row		= $model->getTable('productpropertykeys');
        $groupings	= array();

        $column = 'property_id';
        
        if( $readonly ){
            
            $column = 'property_readonly_id';
        }
        
        foreach($product_ids As $product_id){

            $db->setQuery("Select $column From #__breezingcommerce_products Where id = " . $product_id);
            $property_id = $db->loadResult();

            $db->setQuery("Select id, ordering From #__breezingcommerce_property_keys Where property_id = " . $property_id);
            $property_key_ids = $db->loadAssocList();

            foreach($property_key_ids As $property_key_id){

                $db->setQuery("Select count(id) From #__breezingcommerce_product_property_keys Where product_id = ".intval($product_id)." And property_key_id = " . $property_key_id['id']);
                $haskeys = $db->loadResult();

                if(!$haskeys) {
                    $db->setQuery("Insert Into #__breezingcommerce_product_property_keys (`product_id`,`property_id`,`property_key_id`,published,ordering) Values (".$product_id.",$property_id,".$property_key_id['id'].",$published,".$property_key_id['ordering'].")");
                    $db->query();
                    $keys_updated++;
                }

                $db->setQuery("Select id, ordering From #__breezingcommerce_property_values Where key_id = " . $property_key_id['id']);
                $vals = $db->loadAssocList();

                foreach($vals As $val)
                {
                    $db->setQuery("Select count(id) From #__breezingcommerce_product_property_values Where product_id = ".intval($product_id)." And property_value_id = " . $val['id'] );
                    $hasvals = $db->loadResult();
                    if(!$hasvals)
                    {
                        $db->setQuery("Insert Into #__breezingcommerce_product_property_values (`product_id`,`property_id`,`property_key_id`,`property_value_id`,published,ordering) Values (".$product_id.",$property_id,".$property_key_id['id'].",".$val['id'].",$published,".$val['ordering'].")");
                        $db->query();
                        $values_updated++;
                    }
                }

                if($reorder)
                {
                    $model->getTable('productpropertykeys')->reorder('property_id = '.$property_id.' And product_id = ' . $product_id);
                    $model->getTable('productpropertyvalues')->reorder('property_id = '.$property_id.' And property_key_id = '.$property_key_id['id'].' And product_id = ' . $product_id);
                }
            }
        }

        return JText::_('COM_BREEZINGCOMMERCE_KEYS_UDPATED') . ': ' . $keys_updated . ', ' . JText::_('COM_BREEZINGCOMMERCE_VALUES_UDPATED') . ': ' . $values_updated;
    }
}
