<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access
defined('_JEXEC') or die('Restricted access');

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcPaymentPlugin.php');

interface CrBcPaymentSitePlugin extends CrBcPaymentPlugin{
    /**
     * Init will be called by the order details model
     * to deal with interface methods.
     */
    function init();

    /**
     * Verifies a payment
     * 
     * @param CrBcCart $cart_instance
     * @param stdClass $order_object
     */
    function verifyPayment(CrBcCart $cart_instance, stdClass $order_object);
    
    /**
     * Will be called right after a successfully verified payment.
     * Best is to store the tx id from a payment gateway into a private class variable
     * within verifyPayment() and return it in this method.
     */
    function getPaymentTransactionId();
    
    /**
     * Should return the payment information.
     * The payment information will be visible on invoices and in the personal area of the customer
     * and presented if an order has not the status "paid".
     *
     * The payment information may be a text, containing information how to pay.
     * For instance for bank transfer it would show the payment information.
     * For paypal it would show the vendor's email address to send the money to, and so on...
     *
     * @return string
     */
    function getPaymentInfo();
    /**
     * Will be called in numerous views if an order has been cleared.
     *
     * @return string
     */
    function getAfterPaymentInfo();
    
    /**
     * Return the output to for example redirect to the payment gateway.
     * Will be called right after finishing the checkout process from the user side.
     * Technically, the checkout won't happen here yet but after a successfull payment.
     * 
     * This method will be skipped on manual payments.
     */
    function getInitOutput();
}