<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access
defined('_JEXEC') or die('Restricted access');

abstract class CrBcPlugin
{
    protected $template_vars = array();
    protected $folder = 'site';
    public $tmpl = null;
    public $type = '-none-';
    public $name = '-none-';
    public $template = '';
    
    protected $session_namespace = 'bc_plugin';
    protected $total = null;
    protected $pagination = null;
    protected $data = null;
    protected $limit = 20;
    protected $limitstart = 0;
    protected $filter_order = 'ordering';
    protected $filter_order_Dir = 'asc';
    protected $filter_state = '';
    protected $table = '';

    public $identity_column = 'identity';
    public $detail_view = 'details';

    function __construct(){
        $class = get_class($this);
        $exploded = explode('_', $class);
        $this->type = strtolower($exploded[2]);
        $this->name = strtolower($exploded[3]);
    }
    
    public static function escape($value){
        return htmlentities($value, ENT_QUOTES, 'UTF-8');
    }

    function setDetailsView($toolbarItems = array()){

        $this->template = 'details';
        
        $db = JFactory::getDBO();
        $db->setQuery("Select * From " . $this->table . " Where `" . $this->identity_column . "` = " . $db->Quote(JRequest::getVar($this->identity_column,'0')));
        $row = $db->loadObject();
        $this->assignRef('entity', $row);
    }

    function setListView($toolbarItems = array()){

        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $this->template = 'list';
        $this->session_namespace = $this->type . '_' . $this->name . '_plugin';

        // Get pagination request variables
        $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
        JFactory::getSession()->set('limit', $limit, $this->session_namespace);
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
        JFactory::getSession()->set('limitstart', $limitstart, $this->session_namespace);

        $filter_order     = $mainframe->getUserStateFromRequest(  $option.$this->session_namespace.'_filter_order', 'filter_order', 'ordering', 'cmd' );
        JFactory::getSession()->set('filter_order', $filter_order, $this->session_namespace);

        $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.$this->session_namespace.'_filter_order_Dir', 'filter_order_Dir', 'asc', 'word' );
        JFactory::getSession()->set('filter_order_Dir', $filter_order_Dir, $this->session_namespace);

        $filter_state = $mainframe->getUserStateFromRequest( $option.$this->session_namespace.'_filter_state', 'filter_state', '', 'word' );
        JFactory::getSession()->set('filter_state', $filter_state, $this->session_namespace);

        $lists['order_Dir'] = $filter_order_Dir;
        $lists['order'] = $filter_order;
        $lists['state']	= JHTML::_('grid.state', $filter_state );
        $lists['limitstart'] = $limitstart;

        $ordering = ($lists['order'] == 'ordering');

        $this->assignRef('ordering', $ordering);
        $this->assignRef('lists', $lists);
        $this->assignRef('pagination', $this->getPagination());
        $this->assignRef('items', $this->getData());
    }

    public function getTable($alt_table_classname = '', $alt_table_path = ''){
        jimport( 'joomla.filesystem.file' );

        $table = null;

        if($alt_table_path == '' || $alt_table_classname == '') {
            $table_path = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'plugins' . DS . $this->type . DS . $this->name . DS . 'tables' . DS . 'table.php';
            
            if(JFile::exists($table_path)) {
                require_once($table_path);
                $class = 'CrBc_Plugins_'.ucfirst($this->type).'_'.ucfirst($this->name);
                if(class_exists($class)) {
                    $table = new $class($this->table);
                }
            }
       }
       else {
           if(JFile::exists($alt_table_path)) {
               require_once($alt_table_path);
               $class = $alt_table_classname;
               if(class_exists($class)) {
                   $table = new $class;
               }
           }
      }

      return $table;
    }

    public function store($alt_table_classname = '', $alt_table_path = '', $alt_reorder_where = ''){

        $table = $this->getTable($alt_table_classname, $alt_table_path);

        if($table instanceof JTable){

            $row = $table;
            $data = JRequest::get( 'post' );

            $data = $this->beforeStore($data);

            if (!$row->bind($data)) {
                //$this->setError(JFactory::getDBO()->getErrorMsg());
                return false;
            }

            if (!$row->check()) {
                //$this->setError(JFactory::getDBO()->getErrorMsg());
                return false;
            }

            $storeRes = $row->store();
            $id = 0;
            
            if (!$storeRes) {
                //$this->setError(JFactory::getDBO()->getErrorMsg())
                return false;
            }
            else
            {
                if( intval($data[$this->identity_column]) == 0 )
                {
                    $data[$this->identity_column] = JFactory::getDbo()->insertid();
                }
            }

            if(isset($row->ordering)){
                $row->reorder($alt_reorder_where);
            }

            $this->afterStore($data);
        }

        return $data;
    }

    /**
     * Override this if you need to alter the $data array from the post
     *
     * @param array $data
     * @return array The altered $data
     */
    public function beforeStore($data){
        return $data;
    }

    /**
     * Override this for actions after the storage
     *
     * @param array $data
     * @return void
     */
    public function afterStore($data){

    }
    
    public function beforeDeleteRow($id){
        
    }
    
    public function beforeDeleteRows($ids){
        
    }
    
    public function afterDeleteRow($id){
        
    }
    
    public function afterDeleteRows($ids){
        
    }

    private function createTmplInstance(){
        
        if($this->tmpl === null){
            $libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
            require_once($libpath . 'CrBcPluginTemplate.php');
            $this->tmpl = new CrBcPluginTemplate();
            $array = JRequest::getVar('cid',  0, '', 'array');
            $this->assignRef('plugin_id', (int)$array[0]);
            //$this->assignRef('task', 'edit');
        }
    }

    protected function assignRef($name, $value) {
        $this->createTmplInstance();
        $this->tmpl->assignRef($name, $value);
    }

    public function getHtml(){

        // called to keep the right $this->template defintion order
        $this->display();

        $c = '';
        jimport( 'joomla.filesystem.file' );
        $tmpl_path = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'plugins' . DS . $this->type . DS . $this->name . DS . $this->folder . DS . 'tmpl' . DS . $this->template .'.php';
        
        if(JFile::exists($tmpl_path)){
            $this->createTmplInstance();
            
            $c = $this->tmpl->getTemplate($tmpl_path);
        }
        return $c;
    }
    
    public static function getPlugins($type){
        
        static $plugins_return = array();
        
        if(!isset($plugins_return[$type])){
            
            $db = JFactory::getDbo();
            $db->setQuery("Select * From #__breezingcommerce_plugins Where `type` = ".$db->quote($type)." And published = 1 Order By ordering");
            $plugins = $db->loadObjectList();
            
            $plugins_return[$type] = $plugins;
        }
        
        return $plugins_return[$type];
    }

    /**
     * Override with your implementation
     *
     * $subject is usually used to retrieve the object to work on (e.g. CrBcCart)
     *
     * @param mixed $subject (optional)
     */
    public function init($subject = null){
        
    }

    public static function getPluginInstance($name, $type, $folder = 'admin'){
        static $instances;

        if(!isset($instances[$name.$type.$folder])){
            $instances[$name.$type.$folder] = null;
        }

        if(!($instances[$name.$type.$folder] instanceof CrBcAdminPlugin) && !($instances[$name.$type.$folder] instanceof CrBcSitePlugin)){
            
            jimport('joomla.filesystem.folder');
            jimport('joomla.filesystem.file');

            $path = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'plugins' . DS . $type . DS . $name . DS . $folder . DS;
            if (JFolder::exists($path)) {
                if (JFile::exists($path . 'index.php')) {
                    $libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
                    require_once($libpath . 'CrBcAdminPlugin.php' );
                    require_once($libpath . 'CrBcSitePlugin.php' );
                    require_once($libpath . 'CrBc' . ucfirst($type) . 'Plugin.php' );
                    require_once($path . 'index.php');
                    $class = 'CrBc_Plugins_' . ucfirst($type) . '_' . ucfirst($name) . '_' . ucfirst($folder);
                    if (class_exists($class)) {
                        $plugin = new $class;
                        if ($plugin instanceof CrBcAdminPlugin || $plugin instanceof CrBcSitePlugin) {
                            $instances[$name.$type.$folder] = $plugin;
                            return $plugin;
                        }
                    }else{
                        throw new CrBcCartException('Plugin admin or site class could not be found. Please check the classname.');
                    }
                }
            }
            return null;
        }

        return $instances[$name.$type.$folder];
    }

    public static function getPluginClass($name, $type, $folder = 'admin'){
        static $instances2;

        if(!isset($instances2[$name.$type.$folder])){
            $instances2[$name.$type.$folder] = null;
        }

        if(!$instances2[$name.$type.$folder]){

            jimport('joomla.filesystem.folder');
            jimport('joomla.filesystem.file');

            $path = JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'plugins' . DS . $type . DS . $name . DS . $folder . DS;
            if (JFolder::exists($path)) {
                if (JFile::exists($path . 'index.php')) {
                    $libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
                    require_once($libpath . 'CrBcAdminPlugin.php' );
                    require_once($libpath . 'CrBcSitePlugin.php' );
                    require_once($libpath . 'CrBc' . ucfirst($type) . 'Plugin.php' );
                    require_once($path . 'index.php');
                    $class = 'CrBc_Plugins_' . ucfirst($type) . '_' . ucfirst($name) . '_' . ucfirst($folder);
                    if (class_exists($class)) {
                        $instances2[$name.$type.$folder] = $class;
                        return $instances2[$name.$type.$folder];
                    }
                }
            }
            return null;
        }

        return $instances2[$name.$type.$folder];
    }
    
    /*
     * This should be available across all plugins
     */
    function disableOrdersTax($order_id){
        
        $db = JFactory::getDbo();
        $db->setQuery("Update #__breezingcommerce_orders Set tax_disabled = 1 Where id = " . $db->quote($order_id));
        $db->execute();
    }
    
    /*
     * This should be available across all plugins
     */
    function enableOrdersTax($order_id){
        
        $db = JFactory::getDbo();
        $db->setQuery("Update #__breezingcommerce_orders Set tax_disabled = 0 Where id = " . $db->quote($order_id));
        $db->execute();
    }
    
    /*
     * This should be available across all plugins
     */
    function logInvoice($trans_key, $message, $order_id, $plugin = ''){
        
        require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'CrBcCart.php');
        
        // since it is invoice related, we have to adjust the language according to the settings
        $config = CrBcHelpers::getBcConfig();
        
        // make sure to only load the default backend language
        $lang_tmp = JFactory::getLanguage()->getTag();
        
        switch($config->get('invoice_language','backend')){
            case 'backend':
                JFactory::getLanguage()->setLanguage(JComponentHelper::getParams('com_languages')->get('administrator'));
                break;
            case 'frontend':
                JFactory::getLanguage()->setLanguage(JComponentHelper::getParams('com_languages')->get('site'));
                break;
            case 'frontend_user':
                JFactory::getLanguage()->setLanguage(JFactory::getLanguage()->getTag());
                break;
        }
        
        // write the log with the active language
        CrBcCart::log('plugin_invoice', JText::_($trans_key) . $message, $order_id, $plugin);
        
        JFactory::getLanguage()->setLanguage($lang_tmp);
    }
    
    public function isTaxDisabled(){
        
        return false;
    }

    /**
     * Handles the display for wether admin or site.
     *
     * @return string
     */
    function display(){
        
    }
}