<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class BreezingcommerceModelPlugins extends JModelLegacy
{
    /**
     * Items total
     * @var integer
     */
    private $_total = null;

    /**
     * Pagination object
     * @var object
     */
    private $_pagination = null;

    function  __construct($config) {
        parent::__construct($config);

        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        // Get pagination request variables
        $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);

        $filter_order     = $mainframe->getUserStateFromRequest(  $option.'plugins_filter_order', 'filter_order', 'ordering', 'cmd' );
        $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'plugins_filter_order_Dir', 'filter_order_Dir', 'asc', 'word' );

        $this->setState('plugins_filter_order', $filter_order);
        $this->setState('plugins_filter_order_Dir', $filter_order_Dir);

        $filter_state = $mainframe->getUserStateFromRequest( $option.'plugins_filter_state', 'filter_state', '', 'word' );
        $this->setState('plugins_filter_state', $filter_state);

        $type_state = $mainframe->getUserStateFromRequest( $option.'plugins_type_state', 'type_state', '', 'cmd' );
        $this->setState('plugins_type_state', $type_state);


        

    }

    function getTypes(){
        $this->_db->setQuery("Select Distinct `type` From #__breezingcommerce_plugins");
        return $this->_db->loadObjectList();
    }

    function setPublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_plugins '.
                '  Set published = 1 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();

    }

    function setUnpublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_plugins '.
                '  Set published = 0 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();
    }

    /*
     *
     * MAIN LIST AREA
     * 
     */

    /**
     * @return string The query
     */
    private function _buildQuery(){
        $where = '';
        $filter_state = '';
        if($this->getState('plugins_filter_state') == 'P' || $this->getState('plugins_filter_state') == 'U')
        {
            $published = 0;
            if($this->getState('plugins_filter_state') == 'P')
            {
                $published = 1;
            }

            $filter_state .= ' published = ' . $published;
        }

        if($this->getState('plugins_type_state') != ''){
            $filter_state .= ($filter_state!=''?' And ':'') . ' `type` = ' . $this->_db->Quote($this->getState('plugins_type_state')) . ' ';
        }

        if($filter_state != '')
        {
            $where = ' Where ';
        }

        return 'Select SQL_CALC_FOUND_ROWS * From #__breezingcommerce_plugins ' . $where . ' ' . $filter_state . ' Order By `type`,`ordering`';
    }

    /**
    * Gets the plugins
    * @return array List of plugins
    */
    function getData()
    {
        // Lets load the data if it doesn't already exist
        if (empty( $this->_data ))
        {
            $query = $this->_buildQuery();
            $this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
        }

        return $this->_data;
    }

    function getTotal() {
        // Load the content if it doesn't already exist
        if (empty($this->_total)) {
            $query = $this->_buildQuery();
            $this->_total = $this->_getListCount($query);
        }
        return $this->_total;
    }

    function getPagination() {
        // Load the content if it doesn't already exist
        if (empty($this->_pagination)) {
            jimport('joomla.html.pagination');
            $this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }


    function reorderPlugins($type){
        $row = $this->getTable('plugin');
        $row->reorder("`type` = " . $this->_db->Quote($type));
    }

    function saveOrder()
    {
        $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);

        $total		= count( $items );
        $row		= $this->getTable('plugin');
        $groupings	= array();

        $order		= JRequest::getVar( 'order', array(), 'post', 'array' );
        JArrayHelper::toInteger($order);

        // update ordering values
        for( $i=0; $i < $total; $i++ ) {
            $row->load( $items[$i] );
            $groupings[] = $row->type;
            if ($row->ordering != $order[$i]) {
                $row->ordering = $order[$i];
                if (!$row->store()) {
                    $this->setError($row->getError());
                    return false;
                }
            } // if
        } // for

        $groupings = array_unique( $groupings );
        foreach ($groupings as $group) {
            $row->reorder('`type` = '. $this->_db->Quote($group));
        }
    }

}
