<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcAShippingSitePlugin.php');
require_once($libpath . 'CrBcShippingSitePlugin.php');

class CrBc_Plugins_Shipping_Flatrate_Site extends CrBcAShippingSitePlugin implements CrBcShippingSitePlugin
{
    private $cart = null;
    
    public function  __construct() {
        
        require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'CrBcCart.php');
        
        // always call the parent constructor and always call it _first_
        parent::__construct();
        // define the default table for built-in list/details view
        $this->table = '#__breezingcommerce_plugin_flatrate';
    }
    
    function init($subject = NULL){
        if( !($subject instanceof CrBcCart) ){ throw new Exception('init subject must be of type CrBcCart'); }
        $this->cart = $subject;
    }
    
    function getOptionElementName(){
        return 'crbc_shipping_flatrate';
    }
    
    public function getPluginDisplayName(){
        return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE');
    }
    
    function validateSelectedOption($option, $order_id){
        if(intval($option) <= 0){
            return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_SHIPPING_PLEASE_SELECT_AN_OPTION');
        }
        return '';
    }
    
    function onOptionUpdate($option, $order_id){
        // nothing required here
    }
    
    function determineBestShippingOptions(){
        // nothing, flatrate doesn't care
    }
    
    function onDisplayOptions($selected_key){
        
        $total_net = 0;
        
        try{
            
            $total_net = CrBcCart::getOrderInfoFromCart()['info']->price_net / $this->cart->currency_conversion_rate;
        
        } catch(Exception $e){
            
            return '';
        }
        
        $db = JFactory::getDBO();
        $db->setQuery("Select * From #__breezingcommerce_plugin_flatrate Where published = 1 Order By `rate`");
        $options = $db->loadAssocList();
        
        if(count($options)){
            
            $new_options = array();
            
            foreach($options As $option){
                
                if( ($option['from_total_net'] <= $total_net) && ($total_net <= $option['to_total_net'] || ( $total_net > $option['to_total_net'] && $option['to_total_net'] == 0 ) ) ){
                    
                    $new_options[] = $option;
                }
            }
            
            if( count( $new_options ) > 1 ){
            
                $out = '<select name="'.$this->getOptionElementName().'">'."\n";
                foreach($new_options As $option){

                    CrBcCart::$exclude_from_no_tax_list[] = $option['producttaxclass_id'];
                    $result = CrBcCart::getPrice($option['rate'], $this->cart->currency_conversion_rate, $option['producttaxclass_id'], 1, $this->cart->cart['customer_id']);
                    
                    $translated_object = CrBcHelpers::loadTranslation($option['identity'], 'plugin_shipping_flatrate');
                    if($translated_object !== null){
                        $option['key'] = $translated_object->title;
                    }

                    $out .= '<option value="'.$option['identity'].'"'.($option['identity'] == $selected_key ? ' selected="selected"' : '').'>'.$option['key'].' ('.$this->cart->formatPrice($result['gross']).')</option>'."\n";
                }
                return $out."\n".'</select>'."\n";
                
            }
            else
            if( count( $new_options ) == 1 ){
                
                CrBcCart::$exclude_from_no_tax_list[] = $new_options[0]['producttaxclass_id'];
                $result = CrBcCart::getPrice($new_options[0]['rate'], $this->cart->currency_conversion_rate, $new_options[0]['producttaxclass_id'], 1, $this->cart->cart['customer_id']);
                
                $translated_object = CrBcHelpers::loadTranslation($new_options[0]['identity'], 'plugin_shipping_flatrate');
                if($translated_object !== null){
                    $option['key'] = $translated_object->title;
                }
                
                $out = $new_options[0]['key'].' ('.$this->cart->formatPrice($result['gross']).')';
                $out .= '<div><i class="crbc-fa crbc-fa-truck"></i> '.JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_BASED_ON').'</div>';
                return $out . '<input type="hidden" value="'.$new_options[0]['identity'].'" name="'.$this->getOptionElementName().'" />';
            }

            
        }

        return '';
    }
    
    function getProductTaxclassId($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select producttaxclass_id From #__breezingcommerce_plugin_flatrate Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       }
       return 0;
    }
    
    function getPrice($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select rate From #__breezingcommerce_plugin_flatrate Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result * $this->cart->currency_conversion_rate;
       }
       return 0;
    }
    
    function getName($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select `key` From #__breezingcommerce_plugin_flatrate Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       }
       return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_UNKNOWN');
    }
}