<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcAThemeSitePlugin.php');
require_once($libpath . 'CrBcThemeSitePlugin.php');

class CrBc_Plugins_Theme_Marty_Site extends CrBcAThemeSitePlugin implements CrBcThemeSitePlugin
{
    
    function __construct() {
        parent::__construct();
    }
    
    /**
     * wireUp must be implemented
     * and should set the CSS/JS you want to use
     */
    public function wireUp(){
        
        // IMPORTANT:
        //
        // A breezingcommerce theme does not account for layouts but for styles and javascript, only
        // You can use it to override default styles or your template
        // If you want to override the layout and the markup, please use Joomla's template overrides.
        // If you use template overrides, make sure you deeply test everything as breezingcommerce uses 
        // a lot of dynamic ajax updates on certain elements
        
        // Also important is that you never address bootstrap classes directly.
        // Only style breezingcommerce classes which are prefixed crbc-
        // The prefixed classes are often named after their bootstrap counterparts.
        // e.g. .well would be .crbc-well and so on
        // With upcoming bootstrap updates in Joomla we will try to keep up these classnames such
        // that theme updates will be as much painless as possible.
        
        // request what CSS you want to use
        // the requested CSS must be named after the controller you want to address
        // with the exception of the global css
        $this->setGlobalCss();
        
        // analogue to CSS, you can do the same with javascript and add your own js
        $this->setGlobalJs();
        
        // as example for controllers, the styles are called like using the pattern:
        // $this->set*CONTROLLERNAME*Css
        // Where the first character must be in uppercase
        // this will automatically call the right css file that must be located in
        // /media/breezingcommerce/plugins/theme/*THEMENAME*/css/*CONTROLLERNAME*.css
        // the css file must follow exactly the controller name (lowercase)
        
        // this would e.g. trigger to load the file 
        // /media/breezingcommerce/plugins/theme/*THEMENAME*/css/product.css
        // such that you can style the product page
        $this->setProductCss();
        
        // another example: this would try to load the 
        // styles for the category controller (see a full list of available controllers in the end)
        // $this->setCategoryCss();
        
        // calling custom javascript for the controller is done almost the same
        // this time, the files must be located in 
        // /media/breezingcommerce/plugins/theme/*THEMENAME*/js/*CONTROLLERNAME*.js
        $this->setProductJs();
        
        // if you mistype the controller or there is no such css file, the theme will silently skip its inclusion.
        // so you have to check right in the controller if it is loaded correctly.
        
        // there is one exception: if you request the glocal css and it's not found, it will throw an exception
    
        // TIP if you are triggering your own files from within wireUp(): 
        // use CrBcHelpers::addCssFile($path) instead of Joomla's method. 
        // That will enable you to override style w/o the use of !important
        // Exception on !important: Using CrBcHelpers::addCssFile($path) won't be rendered at last. So if you use breezingcommerce
        // related modules and you want to override THEIR styles, then you might need to use !important as they will be called after the components styles
        
        // There are currently the following controllers you can address:
        // 
        // cart
        // categories
        // categoriespicker
        // category
        // checkout
        // customerprofile
        // files
        // invoices
        // product
        //
        // New controllers might be added at any time. To check what's new, look into the folder /components/com_breezingcommerce/controllers/
    }
}